import boto3


class EC2Manager(object):
    def __init__(self):
        self.cli = boto3.client('ec2')

    def list_security_groups(self):
        '''
        Describes the specified security groups or all of your security groups.

        :return: SecurityGroups
        '''
        return self.cli.describe_security_groups()['SecurityGroups']

    def list_vpcs(self):
        '''
        List one or more of your VPCs.
        :return: Vpcs
        '''
        return self.cli.describe_vpcs()['Vpcs']

    def list_subnet(self):
        '''
        List one or more of your Subnets.

        :return: Subnets
        '''
        return self.cli.describe_subnets()['Subnets']

    def get_sg_id(self, group_name: str):
        '''
        Retrieve subnet id from group name.

        :param group_name: group name
        :return: subnet id
        '''
        return next(
            (i['GroupId'] for i in self.list_security_groups()
             if group_name.lower() in i['GroupName'].lower()), None)

    def get_vpc_id(self, vpc_name: str):
        '''
        Retrieve vpc id from vpc name.
        :param vpc_name: vpc name
        :return:
        '''
        return next(i['VpcId'] for i in self.list_vpcs() if 'Tags' in i
                    for t in i['Tags'] if vpc_name.lower() in t['Value'].lower())

    def get_subnet_id(self, vpc_id: str, subnet_name: str):
        '''
        Retrieve subnet id from vpc id and subnet name.
        :param vpc_id: vpc_id
        :param subnet_name: subnet_name
        :return:
        '''
        return next(s['SubnetId'] for s in self.list_subnet() if vpc_id == s['VpcId'] and 'Tags' in s
                    for t in s['Tags'] if subnet_name == t['Value'])



import asyncio

import aiohttp
import nest_asyncio


class AsyncCrawler(object):

    def crawl_urls(self, urls: list):
        '''
        Retruns crawled htmls from urls using async io.

        :param urls: http url list
        :return: html list
        '''

        try:
            loop = asyncio.get_event_loop()
            return loop.run_until_complete(self._fetch_pages(urls))
        except RuntimeError:
            nest_asyncio.apply()
            return loop.run_until_complete(self._fetch_pages(urls))

    async def _fetch_pages(self, urls: list):
        async with aiohttp.ClientSession() as session:
            return await asyncio.gather(
                *[self._fetch_page(session, url) for url in urls], return_exceptions=True)

    async def _fetch_page(self, session, url: str):
        async with session.get(url) as response:
            html = await response.text()
            return html if response.status == 200 else None



import boto3


class IAMManager(object):
    def __init__(self):
        self.cli = boto3.client('iam')

    def get_role(self, role_name):
        '''
        Retrieves information about the specified role.

        :param role_name: role name
        :return:
        '''
        return self.cli.get_role(RoleName=role_name)['Role']

    def get_role_arn(self, role_name):
        '''
        Retrieves role ARN.

        :param role_name: role name
        :return:
        '''
        return self.get_role(role_name)['Arn']

    def list_role_policies(self, role_name):
        '''
        Lists the names of the inline policies that are embedded in the specified IAM role.

        :param role_name: role name
        :return:
        '''
        return self.cli.list_attached_role_policies(RoleName=role_name)['AttachedPolicies']

    def attach_group_policy(self, group_name, policy_arn):
        '''
        Attaches the specified managed policy to the specified IAM group.

        :param group_name: group name
        :param policy_arn: policy arn
        :return:
        '''
        return self.cli.attach_group_policy(GroupName=group_name, PolicyArn=policy_arn)

    def detach_group_policy(self, group_name, policy_arn):
        '''
        Removes the specified managed policy from the specified IAM group.

        :param group_name: group name
        :param policy_arn: policy arn
        :return:
        '''
        return self.cli.detach_group_policy(GroupName=group_name, PolicyArn=policy_arn)

    def list_group_policies(self, user_group_name, max_items=100):
        '''
        Lists the names of the inline policies that are embedded in the specified IAM group.

        :param user_group_name: user group name
        :param max_items: max items, default=100
        :return:
        '''
        return self.cli.list_group_policies(GroupName=user_group_name, MaxItems=max_items)



import os
from pathlib import Path

import boto3
import fire

from baram.iam_manager import IAMManager
from baram.log_manager import LogManager


class GlueManager(object):
    def __init__(self, s3_path):
        self.logger = LogManager.get_logger()
        self.cli = boto3.client('glue')
        self.im = IAMManager()

        self.worker_type = 'G.1X'
        self.workers_num = 2
        self.timeout = 2880
        self.max_concurrent_runs = 123
        self.max_retries = 0
        self.python_ver = '3'
        self.glue_ver = '3.0'
        self.s3_path = s3_path if 's3://' in s3_path else f's3://{s3_path}'

        # See https://docs.aws.amazon.com/glue/latest/dg/aws-glue-programming-etl-glue-arguments.html
        self.default_args = {
            '--job-language': 'scala',
            '--TempDir': os.path.join(self.s3_path, 'temp'),
            '--enable-continuous-cloudwatch-log': 'true',
            '--enable-glue-datacatalog': 'true',
            '--enable-job-insights': 'true',
            '--enable-metrics': 'true',
            '--enable-spark-ui': 'true',
            '--job-bookmark-option': 'job-bookmark-enable',
            '--spark-event-logs-path': os.path.join(self.s3_path, 'events/'),
            '--encryption-type': 'sse-kms'
        }

    def _get_command(self, name):
        return {
            'Name': 'glueetl',
            'ScriptLocation': os.path.join(self.s3_path, "scripts", f'{name}.scala'),
            'PythonVersion': self.python_ver
        }

    def create_job(self, name, package_name, role_name, extra_jars, security_configuration):
        self.default_args['--class'] = f'{package_name}.{name}'
        self.default_args['--extra-jars'] = extra_jars

        return self.cli.create_job(
            Name=name,
            Description='',
            Role=self.im.get_role_arn(role_name),
            ExecutionProperty={
                'MaxConcurrentRuns': self.max_concurrent_runs
            },
            Command=self._get_command(name),
            DefaultArguments=self.default_args,
            MaxRetries=self.max_retries,
            Timeout=self.timeout,
            SecurityConfiguration=security_configuration,
            GlueVersion=self.glue_ver,
            NumberOfWorkers=self.workers_num,
            WorkerType=self.worker_type
        )

    def get_job(self, job_name):
        return self.cli.get_job(JobName=job_name)

    def update_job(self, name, package_name, role_name, extra_jars, security_configuration):
        self.default_args['--class'] = f'{package_name}.{name}'
        self.default_args['--extra-jars'] = extra_jars

        return self.cli.update_job(
            JobName=name,
            JobUpdate={
                'Role': self.im.get_role_arn(role_name),
                'ExecutionProperty': {
                    'MaxConcurrentRuns': self.max_concurrent_runs
                },
                'Command': self._get_command(name),
                'DefaultArguments': self.default_args,
                'MaxRetries': self.max_retries,
                'Timeout': self.timeout,
                'WorkerType': self.worker_type,
                'NumberOfWorkers': self.workers_num,
                'SecurityConfiguration': security_configuration,
                'GlueVersion': self.glue_ver
            }
        )

    def delete_job(self, name):
        return self.cli.delete_job(JobName=name)

    def delete_table(self, db_name, name):
        try:
            return self.cli.delete_table(
                DatabaseName=db_name,
                Name=name
            )
        except Exception as e:
            self.logger.error(str(e))

    def get_table(self, db_name, name):
        return self.cli.get_table(
            DatabaseName=db_name,
            Name=name
        )

    def refresh_job(self,
                    code_path: str,
                    exclude_names: list,
                    package_name,
                    role_name,
                    extra_jars,
                    security_configuration):
        response = self.cli.list_jobs(MaxResults=1000)
        glue_jobs = set([f'{jn}.scala' for jn in response['JobNames']])
        git_jobs = set([f for f in os.listdir(code_path)])

        rest_in_glue = glue_jobs - git_jobs
        for f in rest_in_glue:
            name = Path(f).stem
            self.delete_job(name)
            self.logger.info(f'{name} deleted.')

        rest_in_git = git_jobs - glue_jobs
        for f in rest_in_git:
            name = Path(f).stem
            if name in exclude_names:
                continue
            self.create_job(name, package_name, role_name, extra_jars, security_configuration)
            self.logger.info(f'{name} created.')


if __name__ == '__main__':
    fire.Fire(GlueManager)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['baram']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.1,<4.0.0',
 'boto3==1.18.65',
 'fire>=0.4.0,<0.5.0',
 'nest-asyncio>=1.5.5,<2.0.0',
 'pytest-cov>=3.0.0,<4.0.0',
 'toml>=0.10.2,<0.11.0']

setup_kwargs = {
    'name': 'baram',
    'version': '0.1.5',
    'description': 'AWS Framework for python',
    'long_description': '## Baram\n\nCloud Framework for AWS Framework.\n\nBaram means "wind" in Korean which makes cloud move conveniently.  \n\n## For Framework Developer\n\nYou can build and install the package as below.\n\n```commandline\n\n$ ./build.sh\n```',
    'author': 'Kwangsik Lee',
    'author_email': 'lks21c@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.10,<4.0.0',
}


setup(**setup_kwargs)
